﻿Imports System.Threading

Public Class frmMain
    '====================================
    ' Invokeメソッドで使用するデリゲート
    '====================================
    Delegate Sub txtMessage_Text_Delegate(ByVal value As String)
    Delegate Sub txtMessage_Scroll_Delegate()
    Delegate Sub btnCardID_Enable_Delegate(ByVal value As Boolean)

    '===============================
    ' txtMessage にメッセージを追加
    '===============================
    Private Sub txtMessage_Text(ByVal value As String)
        txtMessage.Text += value
    End Sub

    '=====================================
    ' txtMessage の表示位置を最終行に移動
    '=====================================
    Private Sub txtMessage_Scroll()
        txtMessage.SelectionStart = txtMessage.Text.Length
        txtMessage.Focus()
        txtMessage.ScrollToCaret()
    End Sub

    '====================================
    ' btnCardID ボタンの有効・無効を設定
    '====================================
    Private Sub btnCardID_Enable(ByVal value As Boolean)
        btnCardID.Enabled = value
    End Sub

    '=============================
    ' 別スレッドでPC/SC通信を実行
    '=============================
    ' clsWinSCardクラスに関して
    ' (1) Timeout_MilliSecond にタイムアウトする時間(ミリ秒)を設定する
    '     設定しなければ、Pasoriにカードがセットされるまで無限に待機する
    ' (2) getCardID() を実行することで、下記のプロパティを読み取れるようになります
    '     CardType … カードの種類
    '     IsFelica … Felicaの場合、True
    '     IDm      … FelicaのIDm
    '     PMm      … FelicaのPMm
    '     IsMifare … Mifareの場合、True
    '     UID      … MifareのUID
    Private Sub Thread_PCSC()
        '----------
        ' 変数定義
        '----------
        Dim pcsc As New clsWinSCard
        Dim msg As String = ""

        '----------------
        ' Delegateの作成
        '----------------
        Dim msg_txt    As New txtMessage_Text_Delegate  (AddressOf txtMessage_Text)
        Dim msg_scroll As New txtMessage_Scroll_Delegate(AddressOf txtMessage_Scroll)
        Dim btn_enable As New btnCardID_Enable_Delegate (AddressOf btnCardID_Enable)

        '----------------
        ' ボタンを無効化
        '----------------
        Me.Invoke(btn_enable, New Object() {False})

        '------------------------------
        ' タイムアウトする時間(ミリ秒)
        '------------------------------
        pcsc.Timeout_MilliSecond = 3000

        '------------------------------------
        ' FelicaのIDm,PMM、MifareのUIDを取得
        '------------------------------------
        If pcsc.getCardID() Then
            If pcsc.IsFelica Then
                msg = "[カードの種類] " + pcsc.CardType + vbNewLine + _
                      "[IDm] " + pcsc.IDm + vbNewLine + _
                      "[PMm] " + pcsc.PMm
            ElseIf pcsc.IsMifare Then
                msg = "[カードの種類] " + pcsc.CardType + vbNewLine + _
                      "[UID] " + pcsc.UID
            End If

            'メッセージを画面に表示
            Me.Invoke(msg_txt, New Object() {msg + vbNewLine})
        Else
            'エラーメッセージを画面に表示
            Me.Invoke(msg_txt, New Object() {pcsc.ErrorMsg + vbNewLine})
        End If

        '--------------------------
        ' メッセージの最終行に移動
        '--------------------------
        Me.Invoke(msg_scroll)

        '----------------
        ' ボタンを有効化
        '----------------
        Me.Invoke(btn_enable, New Object() {True})
    End Sub

    '======================
    ' ボタン・クリック処理
    '======================
    ' 実行中、画面が固まってしまうので実際の処理は別スレッドで実行します
    Private Sub btnCardID_Click(sender As System.Object, e As System.EventArgs) Handles btnCardID.Click
        'PC/SC通信を別スレッドで実行
        Dim th As New Thread(New ThreadStart(AddressOf Thread_PCSC))
        'バックグランドで実行
        th.IsBackground = True
        'スレッド開始
        th.Start()
    End Sub
End Class
